% drfun --- despersion_relation_fun
function ws = drfun(kc)
global  g H rhow h rhoi G nu w w2 ds_option v nut hboundary E1_E2_eta1_eta2 a1_a2_b0_b1
    v = 0.3; % Poisson's ratio
    %  E = G*4(1-v^2)
    w2 = w*w;
    k  = kc(1) + 1i*kc(2);
    Gv = G - 1i*w*nu*rhoi; 
    ve = nu + 1i*G/rhoi/w; 
    N  = w + 2*1i*k^2*ve;    %% 
    
    a  = sqrt(k^2 - 1i*w/ve);
    Sk = sinh(k*h);
    Ck = cosh(k*h);    
    Sa = sinh(a*h);
    Ca = cosh(a*h);                

    J1 = g^2*k^2*Sk*Sa - (N^4 + 16*k^6*a^2*ve^4)*Sk*Sa -8*k^3*a*ve^2*N^2*(Ca*Ck-1); % do not take g^2*k^2 into the parenthesis
    J2 = g*k*(4*k^3*a*ve^2*Sk*Ca + N^2*Sa*Ck - g*k*Sk*Sa);
    Qc = 1 + rhoi/rhow*J1/J2;
    chi=  rhow/rhoi*w2/ve/ve/g/k/tanh(k*H); 
    % select amount of roots you want to observe
    % searching 3 roots is hard. Since the Det =J2*F(Qc), one can search J2
    % and F(Qc) one at a time.   
%% dispersion relation options
%    0: open water
%    1: part of dispersion relation of WS model without J(k)
%    2: J(k) part in dispersion relation of WS model
%    3: simplified dispersion relation of WS model
%    4: dispersion relation of WS model in matrix form
%    5: dispersion relation of WS model derived by Mosig et al. 
%    6: interfacial wave derived in Zhao et al. multi-modes paper
%    7: the Rayleigh-lamb wave equation mentioned in Zhao et al. multi-modes paper
%    8: thin elastic plate model, Fox and Squire's model(1990 deep water, 1994),
%       note that input Young's modulus as G and set nu=0, poission ratio
%       is 0.3, in drfun.m
%    8.1: mass loading model
%    8.5: % finite elastic plate model in Wang and Shen(2010), Eq. (54)
%    9: extended Fox and Squire's model(1994) in Mosig et al. 
%       set nu=0 to reduce to thin elastic plate model,
%       set G=0 and nu=0 to reduce to mass loading model
%    10: RP model(2D thin viscoelastic plate model, Squire and Fox, 1992) in Mosig et al.
%    11: Squire and Allan, 1980, series combination     
%    12: Liu-Mollo-Christensen (1988)  Eq.(A9)
%    13: three-layer model, Zhao and Shen (2018)

    i = ds_option;
    if i ==0 % open water
        ws = w2 - g*k*tanh(k*H);        
    elseif i==1   % WS dispersion relation, one part
        ws = w2-Qc*g*k*tanh(k*H);
    elseif i==2 % WS dispersion relation, the other one part
        ws = J2;
%         ws = J2 *(w2-Qc*g*k*tanh(k*H));
    elseif i==3    % WS dispersion relation, simplify version
        ws = chi*J2*(w2-Qc*g*k*tanh(k*H)); 
    elseif i==4    % WS dispersion relation, implicit form
        M  = (rhow/rhoi-1)*g*k - rhow/rhoi*w2/tanh(k*H);  % g*k*(h/g),  k' = k*h
        W = [ 0,          2*1i*k^2,       a^2+k^2,     0;
            2*1i*k^2*Sk, 2*1i*k^2*Ck,  (a^2+k^2)*Ca,  (a^2+k^2)*Sa;
            N*w*Ck-g*k*Sk,  N*w*Sk-g*k*Ck,  1i*g*k*Ca+2*k*a*w*ve*Sa, 1i*g*k*Sa+2*k*a*w*ve*Ca;
            N*w,  M, -1i*M,  2*k*a*w*ve ];
        ws = det(W);
    elseif i==5  % Mosig et al. derived WS dispersion relation form
%         a  = sqrt(k^2-rhoi*w^2/Gv);       
        Qws = (Sk*Sa*rhoi^4*w^4*(-g^2*k^2)+Sk*Sa*(rhoi^4*w^4*(N^4)+16*a^2*Gv^4*k^6)+8*a*Gv^2*k^3*N^2*rhoi^2*w^2*(1-Ck*Ca))...
            /(g*k*rhoi^2*w^2*(rhoi^2*w^2*Sa*(g*k*Sk-N^2*Ck)+4*a*Gv^2*k^3*Ca*Sk))/rhow*rhoi+1;
        Dws = h/g*((N^2*Ck -g*k*Sk)*Sa -4*a*Gv^2/rhoi^2/w^2*k^3*Ca*Sk); % J2
        ws = (w2-Qws*g*k*tanh(k*H))*Dws;
    elseif i==6 % Zhao, interfacial wave
        r  = (rhow- rhoi)/(rhow+rhoi);
        ws = (w2 - r*g*k) -(w2-g*k)*2*1i*ve*k^2*r/w +4*ve^2*k^3*a*rhoi/(rhoi+rhow);
        % notice: high order term is ignored by timing 0 to the last term
        % in Zhao et al. 2017
    elseif i==7  % the Rayleigh-lamb wave equation, also see formula from https://en.wikipedia.org/wiki/Lamb_waves
        a  = sqrt((rhoi*w^2/G) - k^2);% beta
        b  = sqrt((rhoi*w^2/G)*(1-2*v)/(2-2*v) - k^2);% gamma
        ws = tan(b*h)*(k^2-b^2)^2 + tan(a*h)*4*a*b*k^2; % sysmetic modes
%         ws = tan(a*h)*(k^2-b^2)^2 + tan(b*h)*4*a*b*k^2; % antisymmetric modes
    elseif i ==8 % thin elastic plate model        
        E = G; % Young's modulus
        L = E*h^3/12/(1-v*v); % flexural regidity           
        % deep water, % Fox& squire: 1990, reflection and transmission at shore face ice edge
%         ws =  w^2 - (L*k^5+rhow*g*k)/(rhow*cosh(k*H)+rhoi*k*h); %  
        % no deep water assumption, %Fox& squire: 1994
        ws = rhow*w2 - (L*k^4 + rhow*g - rhoi*h*w2)*k*tanh(k*H);
    elseif i ==8.1 % mass loading model
        ws = rhow*w2 - (rhow*g - rhoi*h*w2)*k*tanh(k*H);        
    elseif i == 8.5 % finite elastic plate model in Wang and Shen(2010), Eq. (54),use with caution, it is dimensionaless
        G = E;
        nu = 0;
        w1 = w*sqrt(h/g);
        Gv = G - 1i*w*nu*rhoi;  
        G1 = Gv/rhow/g/h;
        k1 = k*h;
        ws = w1^2-(G1/3*k1^4+1)*k1*tanh(k*H)/(1+rhoi/rhow*k1*tanh(k*H));                 
    elseif i==9   % FS model (Fox and Squire, 1994, thin visco-elastic plate)
%         Qfs = Gv*h^3/6/rhow/g*(1+v)*k^4 - rhoi*h*w^2/rhow/g + 1; %in Mosig et al. (2015), derived from 3D by J. Mosig
        Qfs = Gv*h^3/6/rhow/g/(1-v)*k^4 - rhoi*h*w^2/rhow/g + 1; % in Li et al. (2015)  (isotropic viscoelastic thin beam, Squire and Allan, 1980) 
        ws = w2-Qfs*g*k*tanh(k*H);
    elseif i==10   % RP model(2D thin viscoelastic plate model, Squire and Fox, 1992) in Mosig et al.
%         gamma is viscosity paprameter which is distinct from the
%         viscosity parameter nu in WS and FS models.
%   gamma = ?
        Qrp = G*h^3/6/rhow/g*(1+v)*k^4 - rhoi*h*w^2/rhow/g + 1 - 1i*w/rhow/g*gamma;
    elseif i==11 % Squire and Allan, 1980, series combination   
        if(~isempty(E1_E2_eta1_eta2))  
            E1 = E1_E2_eta1_eta2(1);
            E2 = E1_E2_eta1_eta2(2);
            X1 = E1_E2_eta1_eta2(3);
            X2 = E1_E2_eta1_eta2(4);
            a1 = E1/X1 + E1/X2 + E2/X2;
            a2 = E1*E2/X1/X2;
            b0 = E1;
            b1 = E1*E2/X2;
        elseif (~isempty(a1_a2_b0_b1))
            a1 = a1_a2_b0_b1(1);
            a2 = a1_a2_b0_b1(2);
            b0 = a1_a2_b0_b1(3);
            b1 = a1_a2_b0_b1(4);
        else
            error('Error for using option11:Squire and Allan(1980), elastic moduli and viscosities must be defined in either a1_a2_b0_b1 or E1_E2_eta1_eta2')
        end
		ws = h^3/6*(b0*w+1i*b1)*k^5+(w-a2/w+1i*a1)*(rhow*g-rhoi*h*w^2)*k-rhow*(w-a2/w+1i*a1)*w^2;
    elseif i == 12   
        nu = 0.; % viscosity
        P = 0; % compressive stress
        % B, Q, M are found from near Eq.(6).
        E = G; % E = G*4(1-v^2)
        L = E*h^3/12/(1-v*v); % flexural regidity    
        B = L/rhow; % E*h^3/12/(1-v^2);
        Q = P*h/rhow;
        M = rhoi*h/rhow;
        n = 1i*w;
        G = g + B*k^4 - Q*k^2;
        ws = abs((n^2*(1+k*M) + k*G)^2 + n*nu*k^2*(n^2*(1+2*k*M) + 2*k*G));
    elseif i==13 %  three-layer model, Zhao and Shen (2018)
% complex viscosity of ice layer
        alpha1 = sqrt ( k^2- w/ve * 1i ); 
        N1     = w + 2* ve * k^2* 1i;
        L1     = 2 * k * alpha1 * w * ve;
        Skh    = sinh(k*h);
        Ckh    = cosh(k*h);
        Sah    = sinh(min(20,alpha1*h));
        Cah    = cosh(min(20,alpha1*h));    
% viscosity of water boundary layer      
        alpha2 = sqrt ( k^2- w/nut * 1i );    % nut or ve???
        N2     = w + 2* nut * k^2* 1i;
        L2     = 2 * k * alpha2 * w * nut;
        Skb    = sinh(k*hboundary);
        Ckb    = cosh(k*hboundary);
        Sab    = sinh(min(20,alpha2*hboundary));
        Cab    = cosh(min(20,alpha2*hboundary));
% water layer
        Sh = sinh(min(20,k*(H-hboundary)));
        Ch = cosh(min(20,k*(H-hboundary)));
        th = Sh/Ch;
        M  = w^2/th;  % this definition is different from viscoelastic model
      
% essemble matrix of determinant
        aa = 0;
        aa(1,1) = 2*1i*k^2*Skh;
        aa(1,2) = 2*1i*k^2*Ckh;
        aa(1,3) = (alpha1^2+k^2)*Cah;
        aa(1,4) = (alpha1^2+k^2)*Sah;

        aa(2,1) = N1*w*Ckh-g*k*Skh;
        aa(2,2) = N1*w*Skh-g*k*Ckh;
        aa(2,3) = 1i*k*g*Cah+L1*Sah;
        aa(2,4) = 1i*k*g*Sah+L1*Cah;

        aa(3,1) = rhoi/rhow*N1*w;
        aa(3,2) =-rhoi/rhow*g*k;
        aa(3,3) = rhoi/rhow*1i*g*k;
        aa(3,4) = rhoi/rhow*L1;
        aa(3,5) =-N2*w;
        aa(3,6) = g*k;
        aa(3,7) =-1i*g*k;
        aa(3,8) =-L2;

        aa(4,2) = rhoi/rhow*ve*2*1i*k^2;
        aa(4,3) = rhoi/rhow*ve*(alpha1^2+k^2);
        aa(4,6) =-nut*2*1i*k^2;
        aa(4,7) =-nut*(alpha2^2+k^2);

        aa(5,1) =-1i*k;
        aa(5,4) =-alpha1;
        aa(5,5) = 1i*k;
        aa(5,8) = alpha2;

        aa(6,2) =-1.;
        aa(6,3) = 1i*1.;
        aa(6,6) = 1.;
        aa(6,7) =-1i*1;

        aa(7,5) = N2*w*Ckb+M*Skb;
        aa(7,6) =-N2*w*Skb-M*Ckb;
        aa(7,7) =-L2*Sab+1i*M*Cab;
        aa(7,8) = L2*Cab-1i*M*Sab;

        aa(8,5) =-2*1i*k^2*Skb;
        aa(8,6) = 2*1i*k^2*Ckb;
        aa(8,7) = (alpha2^2+k^2)*Cab;
        aa(8,8) =-(alpha2^2+k^2)*Sab;
        
        ws = det(aa);
    end
    
    ws = abs(ws);
end