% search general solution for any input dispersion relation  
% (1) initial condition: model parameters
% (2) Four loops (G, nu, h, T):
%     1) search wanue modes in desiged area, call dispersiotexn relation
%     function. nuarious models are edited in dispersion_relation_function file.
%     return searched real roots real, imarginary roots kimag, complex ones kc
%     2) plot 
% (3) output (save or plot)

clc
clear; 
close all
global nu G rhoi rhow g T h H w ds_option v E1_E2_eta1_eta2 a1_a2_b0_b1
%% dispersion relation option
%    1: part of dispersion relation of WS model without J(k)
%    2: J(k) part in dispersion relation of WS model
%    3: simplified dispersion relation of WS model
%    4: dispersion relation of WS model in matrix form
%    5: dispersion relation of WS model derived by Mosig et al. 
%    6: interfacial wave derived in Zhao et al. multi-modes paper
%    7: the Rayleigh-lamb wave equation mentioned in Zhao et al. multi-modes paper
%    8:
%    9: extended Fox and Squire's model in Mosig et al. 
%    10: RP model(2D thin viscoelastic plate model, Squire and Fox, 1992) in Mosig et al.
%    11: Squire and Allan, 1980, series combination. Please define elastic moduli E1, E2 and the viscosities eta1, eta2  as E1_E2_eta1_eta2 = [E1, E2, eta1, eta2] or define the derived variables a1,a2,b0,b1 in a1_a2_b0_b1, according to Eq.(8) of their paper.
% (1)  setting parameters
    H = 100;        % water depth
	g = 9.8;        % gravity accleration
	rhoi = 910;     % density of ice
	rhow = 1025;    % density of water
    H  = 100;       % water depth
    Gn = 1.e9;%0.0%2.18e9;%1.e9;      % elastic modulus
    hn = 0.5;%[0.5,1,1.5];        % ice thickness
    fn = linspace(0.05,1,40);%linspace(1/20,1/0.01,40);
    wn = 2*pi*fn;  % angular frequency
    Tn = 2*pi./wn; % wave period
    nun = 1.e5;%10.0%2.82e8;%1.e5;      % viscosity

    ds_option= 9;     % dispersion relation option, refer to drfun.m
    a1_a2_b0_b1 = [1.4e-2,0.07e-4 1.2e9 1.02e7];
%     a1_a2_b0_b1 = [1e20 100.0 1e20 1e20];
%% (2)  calculation               
for ih = 1:length(hn)    
for inu= 1:length(nun)
for iG = 1:length(Gn)       
    n = 0;  
    kn = [];    % save final wavenumbers in file.
    k0n = [];   % save open water wavenumbers
    k_out = []; % save wavenumber for given T, h, G, nu       
    for iT = 1:length(Tn)        
        T  = Tn(iT);
        w  = 2*pi/T;
        h  = hn(ih); 
        G  = Gn(iG); 
        nu = nun(inu);
        n = n + 1;
%         disp(['w =' num2str(w) ',  h=' num2str(h) ',   G =' num2str(G) ',  nu=' num2str(nu)])  
% 2-1) search specific wanue modes in a region, plot
    % shear wave
        Gv = G - 1i*w*nu*rhoi;        
        ks(n) = w/sqrt(Gv/rhoi); 
    % pressure wave
        v  = 1/3; %Paisson ratio
        lambda= 2*Gv*v/(1-2*v); 
        kp(n) = w/sqrt((2*Gv + lambda)/rhoi);  
    % set searching region 
        xa = 1.e-8;
        ya = xa;
        if nu ==0
            Numy = 1;
            Numx = 10000;
            xb = 5*w^2/g;
            yb = 1.e-4;
            ya = 1.e-8;
            index = 1; % global searching, can detect new solutions but time comsuming
        else
            xb = .1;
            yb = .001;
            Numx = 600;
            Numy = 300;
            index = 0; % track all roots found in the first loop.
        end   
    % start searching
        contourplot = 0;  % =1, show root found by global searching in a contour plot
        [k_out,k0n(n)] = wavemodes_search(index,contourplot,k_out,xa,xb,ya,yb, Numx,Numy);      
        k_out = k_out(abs(k_out-ks(n))>1.e-6); % remove shear wave root  
        k_out = unique(floor(k_out*1.e8)/1.e8);
    % save result
        if index == 0
            kn(n,1:length(k_out)) = k_out;
        else
            kn = [kn; k_out];
        end
% %     % compute k of thin elastic wave mode in Wang and Shen, 2010, Eq.(56)    
% %         kc = fminsearch(@thin_elastic_wavemode,[real(k_out(2)) 0]);
% %         kthin(n) = kc(1) + 1i*kc(2);
% %     % compute k from interfical wave model
% %         kc = fminsearch(@interfacial_wave,[real(k_out(2)) imag(k_out(2))]);
% %         kintf(n) = kc(1) + 1i*kc(2);
  % plot
        figure(11)
        hold on
        f = w; %T;
%         if nu==0
%             xlabel('\omega');
%             ylabel('k_r (m^{-1})');
%             semilogy(f,real(k_out),'.k',f,k0n(n),'.r','MarkerSize',8)
%             ylabel('k_r')
%         else
            subplot(211); 
            if (inu==1)
            plot(f,real(k_out)/k0n(n),'.k','MarkerSize',8)
            end
            if (inu==2)
            plot(f,real(k_out)/k0n(n),'.b','MarkerSize',8) 
            end
            if (inu==3)
            plot(f,real(k_out)/k0n(n),'.r','MarkerSize',8) 
            end
            hold on
           %             plot(f,real(kthin(n))/k0n(n),'og','MarkerSize',8)   
            xlabel('w[rad/s]');%('T (s)');
            ylabel('k_r/k_{ow}');
            subplot(212);
            if(inu==1)
            loglog(f,imag(k_out),'.k','MarkerSize',8); hold on
            end 
            if (inu==2)
            loglog(f,imag(k_out),'.b','MarkerSize',8); hold on  
            end
            if (inu==3)
            loglog(f,imag(k_out),'.r','MarkerSize',8); hold on  
            end
            xlabel('w[rad/s]');%('T (s)');
            ylabel('k_i (m^{-1})');            
        end
        subplot(211);
        title(['T =' num2str(T) 's,  h=' num2str(h) 'm']);   
        %title(['T =' num2str(T) 's,  h=' num2str(h) 'm,   G =' num2str(G,'%10.3e') 'Pa,  nu=' num2str(nu,'%10.3e') 'm/s^2']);                  
        drawnow; 
end
end
end
subplot(212);loglog(wn,0.01*wn.^11)
kn
save('roots_kn.mat');  
knr=real(kn);kni=imag(kn);
figure(20);plot(Tn,knr./k0n')
knrnondim=knr./k0n';
format longEng
knout=[Tn',knrnondim,kni];
xlswrite('results.xls',knout);

% kni1=kniall(1,:);kni2=kniall(2,:);kni3=kniall(3,:);
% figure(1)
% hold on;
% loglog(wn,kni1,'k');
% loglog(wn,kni2,'b');
% loglog(wn,kni3,'r');
% % xticks([0.3,0.4,0.5,1,2,3,4,5]);
% xlabel('\omega [rad/s]');ylabel('q [m^-^1]');
% legend('\nu=0.01 m^2/s','    1 m^2/s','    100 m^2/s');
% hold off;
% 

  subplot(212);loglog(wn,1000*wn.^11)