function [k_out,k0,f1] = wavemodes_search(index,contourplot,kguess,xa,xb,ya,yb,Numx,Numy)  % L-loop variables
%%  1) define a searching region and define grid size
%   2) calculate dispersion relation function on mesh
%   3) plot & save the mesh data for visually searching and checking auto
%      method code
%   4) find local minimas based on the data automatically
%   5) plot roots for double checking & save roots in mat file
%
global options H w g w2 G nu
    options = optimset;
    options = optimset(options,'TolX',1e-18);
    options = optimset(options,'TolFun',1e-18);
    options = optimset(options,'MaxFunEvals',Inf);
    options = optimset(options,'MaxIter',2000);
    options = optimset(options,'Display','off');
    options = optimset(options,'Algorithm','levenberg-marquardt');
    w2 = w*w;
    % open water
    openwater = @(k) w2 - g*k*tanh(k*H); % open water
    k0 = fsolve(openwater,1,options);  % fminsearch failed    
    if length(kguess)==0 || index==1
    % %  1) define a searching region and generate discrete mesh     
        kr = linspace(xa,xb,Numx);
        ki = linspace(ya,yb,Numy);
        f1 = ones(Numx,Numy);    % this method costs too much, but first choice
    %   2) calculate dispersion relation function on mesh
        for i = 1:Numx
        for j = 1:Numy
            f1(i,j)= drfun([kr(i) ki(j)]);
        end
        end    
        %   
%         %  3) plot & save the mesh data
%         %  output .dat file for tecplot
%             str = sprintf('f1.plt');
%             fid1=fopen(str,'w+');
%             fprintf(fid1,'VARIABLES = kr,ki,f1\n');
%             fprintf(fid1,'ZONE i = %d, j =%d \n',Numx,Numy);
%             for j = 1:Numy
%             for i = 1:Numx
%               fprintf(fid1,'%g  %g %g\n',kr(i),ki(j),f1(i,j)); 
%             end
%             end
%             fclose(fid1);    
        % control contour lines number
        if contourplot==1 % show contour plot of roots.
            if nu == 0
                semilogy(kr,f1);
                xlabel('k_r'); ylabel('RF function')
                drawnow
            else
                v =logspace(log10(min(min(f1))),min(9,log10(max(max(f1)))),20);
                contourf(kr,ki,f1',v,'k')
                drawnow
                xlabel('k_r'); ylabel('k_i')
            end
        end
        %   4) find local minimas based on the data automatically   
        %      to find roots of wave number kr and attenuation ki
        mf = imregionalmin(f1);
        [x,y] = find(mf>0);   % indexes local minimas
        y = y(x>1);
        x = x(x>1);        
        for i = 1:length(x)
            k0n(i,:) = [kr(x(i)) ki(y(i))];
        end
    else
        for i = 1:length(kguess)
            k0n(i,:) = [real(kguess(i)) imag(kguess(i))];
        end
    end
    % set sepcific searching rule according to the contour plots/root plotted.
    %     %to search complex k, fminsearch requires to split k into kr & ki       
    k_out =[];       
    for i = 1:size(k0n,1)        
        if nu == 0 % it failed somtimes when using fminsearch to search
            k = fsolve(@drfun,k0n(i,:));      
            k_out = [k_out;  k(1)+ 1i*0];
        else
            k = fminsearch(@drfun,k0n(i,:),options);      
            k_out = [k_out  k(1)+1i*k(2)];
        end        
    end    
%     
end